# -*- coding: utf-8 -*-
from setuphelpers import *

# bin_contains = "ProcessMonitor"
# app_name = "ProcessMonitor"
# app_dir = makepath(programfiles32, app_name)
# tools_list = [
#     "Procmon",
#     "Procmon64",
#     "Procmon64a",
# ]

app_name = "Process Monitor"
zip_name = "ProcessMonitor.zip"
app_dir = makepath(programfiles32, app_name)
app_path = makepath(app_dir, "Procmon.exe")
icon_path = makepath(app_dir, "Procmon.exe")
audit_version_number = False
win64app = None

def get_installed_version(uninstallkey):
    for soft in installed_softwares(uninstallkey):
        return soft.get("version", None)
    return None

def install():
    # Declaring local variables
    package_version = control.get_software_version()
    installed_version = get_installed_version(app_name)

    # Installing software
    if installed_version is None or Version(installed_version) < Version(package_version) or force:
        print("Installing: %s (%s)" % (app_name, control.get_software_version()))
        killalltasks(ensure_list(control.impacted_process))
        if isdir(app_dir):
            remove_tree(app_dir)
        mkdirs(app_dir)
        print("Extracting: %s to: %s" % (zip_name, app_dir))
        unzipped_dir = zip_name.rsplit(".", 1)[0]
        # unzip(zip_name, app_dir)
        unzip(zip_name, unzipped_dir)
        move_directory(unzipped_dir, app_dir)

        # Adding software to regitry
        print("Registering: %s to Windows Registry" % app_name)
        register_windows_uninstall(control, win64app=win64app)
        register_uninstall(app_name, win64app=win64app, install_location=app_dir, icon=icon_path)
    else:
        print("%s is already installed and up-to-date (%s)" % (app_name, package_version))

    # Configure App
    #run('"%s" /accepteula' % app_path) ----> ???
    #run('"%s" /REGISTER ALL' % app_path) ----> ???

    # Creating custom shortcuts
    create_desktop_shortcut(app_name, app_path, icon=icon_path, wDir=r"%userprofile%", arguments=r"/c %appdata%\Cmder")
    create_programs_menu_shortcut(app_name, app_path, icon=icon_path, wDir=r"%userprofile%", arguments=r"/c %appdata%\Cmder")


def uninstall():
    #run_notfatal('"%s" /UNREGISTER ALL' % app_path) ----> ???
    # Uninstalling software
    killalltasks(ensure_list(control.impacted_process))
    if isdir(app_dir):
        remove_tree(app_dir)
    unregister_uninstall(app_name, win64app=win64app)

    # Removing shortcuts
    remove_desktop_shortcut(app_name)
    remove_programs_menu_shortcut(app_name)


def audit():
    # Declaring local variables
    package_version = control.get_software_version()
    installed_version = get_installed_version(app_name)

    # Auditing software
    if installed_version is None:
        print("%s is not installed" % app_name)
        return "ERROR"
    elif Version(installed_version) != Version(package_version) and audit_version_number:
        print("%s is installed in version (%s) instead of (%s)" % (app_name, installed_version, package_version))
        return "WARNING"
    else:
        print("%s (%s) is installed" % (app_name, installed_version))
        return "OK"

def move_directory(source, destination):
    """
    Moves a directory safely, avoiding unnecessary subfolders.

    Args:
        source (str): The path of the directory to move.
        destination (str): The destination path where the directory will be moved to.

    Returns:
        bool: True if the directory was successfully moved, False otherwise.
    """
    try:
        if not os.path.exists(destination):
            os.makedirs(destination)

        for item in os.listdir(source):
            item_path = os.path.join(source, item)
            if os.path.isfile(item_path):
                shutil.move(item_path, destination)
            elif os.path.isdir(item_path):
                new_destination = os.path.join(destination, item)
                move_directory(item_path, new_destination)

        os.rmdir(source)
        return True
    except Exception as e:
        print(f"ERROR: {e}")
        return False